'Geologic Data Assistant
'Version 1.0
'coded by Evan Thoms, U.S. Geological Survey, 2003-2006
'
'Update all lines with Map.Layers(<Layername>) or
'Map.SelectionLayer.Name if the version number changes

'Put this file and the accompanying ArcPad.apx file in the
'../ArcPad/System folder

Option Explicit

' Global variables
Dim g_objDatum, g_strStaID
Dim g_intI

'**************************MAP events************************
Sub CheckTheLayers 'OnBeforeDraw event of the MAP systemobject
'Enables toolbar buttons according to the layers present.
  Dim objCurrLay, objGDAbar', i
  Set objGDAbar = Toolbars("GDA")

  If objGDAbar Is Nothing Then Exit Sub

  'Stations layer
  Set objCurrLay = Map.Layers("Stations_1.0")
  If objCurrLay Is Nothing Then
    objGDAbar.Item("btnAddStation").Enabled = False
  Else	
    objGDAbar.Item("btnAddStation").Enabled = True				
  End If

  'Geolines layer
  Set objCurrLay = Map.Layers("Geolines_1.0")
  If objCurrLay Is Nothing Then
    objGDAbar.Item("btnAddLine").Enabled = False
  Else
    objGDAbar.Item("btnAddLine").Enabled = True
  End If

  'Mylar layer
  Set objCurrLay = Map.Layers("Mylar_1.0")
  If objCurrLay Is Nothing Then
    objGDAbar.Item("btnDigMylar").Enabled = False
  Else
    objGDAbar.Item("btnDigMylar").Enabled = True
    objCurrLay.Properties("ShowEditform") = False
  End If
  objGDAbar.Item("btnStaAtGPS").Enabled = False
End Sub

'--------------------------------------------------------------
Sub CheckForDependents 'OnVertexEditBegin event to check for dependent stations
'before moving
  Dim rsSta

' Bail if we are not dealing with the Stations layer
  If Not LCase(Map.SelectionLayer.Name) = "stations_1.0" Then Exit Sub

' Get the datum X and Y.
  Set rsSta = Map.SelectionLayer.Records
  Set g_objDatum = Application.CreateAppObject("Point")
  rsSta.Bookmark = Map.SelectionBookmark	
  g_objDatum.X = rsSta.Fields.Shape.X 
  g_objDatum.Y = rsSta.Fields.Shape.Y	
  g_strStaID = rsSta.Fields("STAID").Value

' Check for stations dependent on the position of the datum
  rsSta.MoveFirst
  g_intI = 0
  While Not rsSta.EOF
    If rsSta.Fields("Datum").Value = g_strStaID Then
      g_intI = g_intI + 1
    End If
    rsSta.MoveNext
  Wend

' If there are no offset stations, bail
  If g_intI = 0 Then Exit Sub

' Notify the user that other stations will be moved as well if they continue
  If MsgBox (g_intI & " offset stations will also be moved. Continue?", vbYesNo) = vbNo Then 
    Map.SelectionLayer.Properties("featuremoving") = False
    Application.ExecuteCommand("modeselect")
    Exit Sub
  End If
End Sub

'--------------------------------------------------------------
Sub FeatureChangedEvents 'OnFeatureGeometryChanged event. Moves all stations and structural measurements
' that are dependent on the station selected by the same amount that the station has been moved.
  Dim rsSta, rsStruct, objStructLay, strTempStaID
  Dim intDeltaX, intDeltaY, objNewPt

  Set objNewPt = Application.CreateAppObject("Point")

' Bail if it is not the stations layer that is being edited
  If Not LCase(Map.SelectionLayer.Name) = "stations_1.0" Then Exit Sub 

  Set rsSta = Map.SelectionLayer.Records
  rsSta.Bookmark = Map.SelectionBookmark

' Change the HOWLOC field if it was GPS
  If rsSta.Fields("HowLoc").Value = "GPS" Then
    rsSta.Fields("HowLoc").Value = "Edited GPS"
    rsSta.Update
  End If

' Get the delta x and y of the move 
  intDeltaX = rsSta.Fields.Shape.X - g_objDatum.X
  intDeltaY = rsSta.Fields.Shape.Y - g_objDatum.Y

' Apply the delta x and y to all dependent stations
  Set objStructLay = Map.Layers("Structure_1.0")
  rsSta.MoveFirst
  While Not rsSta.EOF
    If rsSta.Fields("DATUM").Value = g_strStaID Then
      objNewPt.X = rsSta.Fields.Shape.X + intDeltaX
      objNewPt.Y = rsSta.Fields.Shape.Y + intDeltaY
      strTempStaID = rsSta.Fields("StaID")
      Set rsSta.Fields.Shape = objNewPt
      rsSta.Update

' This loop is for structural measurements dependent on dependent stations
      If Not objStructLay Is Nothing Then
        objStructLay.Editable = True
        Set rsStruct = objStructLay.Records
        If rsStruct.RecordCount > 0 Then
          rsStruct.MoveFirst
            While Not rsStruct.EOF
              If rsStruct.Fields("STAID").Value = strTempStaID Then
                objNewPt.X = rsStruct.Fields.Shape.X + intDeltaX
                objNewPt.Y = rsStruct.Fields.Shape.Y + intDeltaY
                Set rsStruct.Fields.Shape = objNewPt
                rsStruct.Update
              End If
              rsStruct.MoveNext
            Wend
            strTempStaID = ""
        End If
      End If
    End If
    Map.Layers("Stations_1.0").Editable = True
    rsSta.MoveNext
  Wend
	
' This loop is for structural measurements dependent on datum stations
  If Not objStructLay Is Nothing Then
    objStructLay.Editable = True
    Set rsStruct = objStructLay.Records
    If rsStruct.RecordCount > 0 Then
      rsStruct.MoveFirst
      While Not rsStruct.EOF
        If rsStruct.Fields("STAID").Value = g_strStaID Then
          objNewPt.X = rsStruct.Fields.Shape.X + intDeltaX
          objNewPt.Y = rsStruct.Fields.Shape.Y + intDeltaY
          Set rsStruct.Fields.Shape = objNewPt
          rsStruct.Update
        End If
        rsStruct.MoveNext
      Wend
        strTempStaID = ""
    End If
  End If

' get the new XY of the moved datum station
  rsSta.Bookmark = Map.SelectionBookmark
  g_objDatum.X = rsSta.Fields.Shape.X
  g_objDatum.Y = rsSta.Fields.Shape.Y

  Map.Layers("Stations_1.0").Editable = True
  Application.UserProperties("DATUMID") = ""
End Sub

'--------------------------------------------------------------
Sub WasAStationSelected 'OnSelectionChanged event of the Map systemobject
'to determine if the btnOffsetSta button should be enabled
  Dim objSelLayer

  Set objSelLayer = Map.SelectionLayer
  If objSelLayer Is Nothing Then 
    Application.Toolbars("GDA").Item("btnOffsetSta").Enabled = False
    Exit Sub
  End If
  If LCase(objSelLayer.Name) = "stations_1.0" Then
    Application.Toolbars("GDA").Item("btnOffsetSta").Enabled = True
  Else
    Application.Toolbars("GDA").Item("btnOffsetSta").Enabled = False
  End If
End Sub

'--------------------------------------------------------------
Sub WhichMode 'OnPointerModeChanged event of Map object
	Dim objGDAToolbar
	Set objGDAToolbar = Application.Toolbars("GDA")
	If objGDAToolbar Is Nothing Then Exit Sub
	Dim strCommand, i
	i = 1
	If strCommand <> "modedrawfreehandline" Or strCommand <> "modedrawpoint" Or strCommand <> "addgpspoint" Then
		While i <> Application.Toolbars("GDA").Count
			If Application.Toolbars("GDA").Item(i).Name <> "btnOffsetSta" Then
				Application.Toolbars("GDA").Item(i).Checked = False
			End If
			i = i + 1
		Wend
	End If
End Sub

'--------------------------------------------------------------
Sub FeatAdd 'OnFeatureAdded event of Map object. One way to get GPS averaged
' coordinates since they are not actually written until this event which follows
' editform OnOk event.

' Bail if the selection layer is not the stations layer
  If Not LCase(Map.SelectionLayer.Name) = "stations_1.0" Then Exit Sub

  Dim rsSta, strStaID
  Dim objNewPt
  Dim objStaBM
  Dim objStructLay, rsStruct

' Get the XY of the station location
' Store them explicitly in the dbf
  Set rsSta = Map.SelectionLayer.Records
  rsSta.Bookmark = Map.SelectionBookmark

  rsSta.Fields("MAPX").Value = rsSta.Fields.Shape.X
  rsSta.Fields("MAPY").Value = rsSta.Fields.Shape.Y
  rsSta.Update

' Update any related strucural measurement stations
' During the creation of a new station, structural measurements are added by applying
' the XY of the station to a new point in the structure layer. But for stations
' located by averaged GPS fixes, the averaged XY coordinates are not actually written 
' to the shape field of the selected record until after the editform has been closed. 
' (They are not even written during the OnOk event of the form. That event and the 
' OnFeatureAddded event are not the same.), so structural measurement locations need
' to be updated during the OnFeatureAdded event related to the station. 

' Get the station ID
  strStaID = rsSta.Fields("STAID").Value

' Start updating by advancing through the structure layer recordset
  Set objStructLay = Map.Layers("Structure_1.0")
  If Not objStructLay Is Nothing Then
    objStructLay.Editable = True
    Set rsStruct = objStructLay.Records
    If rsStruct.RecordCount > 0 Then
      rsStruct.MoveFirst
      While Not rsStruct.EOF
        If rsStruct.Fields("STAID").Value = strStaID Then
          Set rsStruct.Fields.Shape = rsSta.Fields.Shape
          rsStruct.Update
        End If
        rsStruct.MoveNext
      Wend
    End If
  End If

' Return the selection to the newly created station
  Map.Layers("Stations_1.0").Editable = True
End Sub

'********************************GPS events***************************
Sub IsGPSFixValid 'OnPosition event of GPS systemobject to determine if the
'btnStaAtGPS button on the GDA toolbar should be enabled
	Dim objGDAToolbar
	Set objGDAToolbar = Application.Applets("GDA")
	If objGDAToolbar Is Nothing Then Exit Sub
	If Application.GPS.IsValidFix = False Then 
		Application.Toolbars("GDA").Item("btnStaAtGPS").Enabled = False
	Exit Sub
	End If
	Application.Toolbars("GDA").Item("btnStaAtGPS").Enabled = True
End Sub

'--------------------------------------------------------------
Sub RememberTheStationID 'OnAverageStop event of GPS systemobject 
	If Map.Layers("Stations_1.0").Editable = True Then
		Map.Layers("Stations_1.0").Forms("Editform").Caption = Application.UserProperties("StationID")
		Application.UserProperties("StationID") = ""
		Map.Layers("Stations_1.0").Forms("Editform").Pages(1).Controls("cboHowLoc").Value = "GPS"
	End If 
End Sub

'--------------------------------------------------------------
Sub GPSInactive 'Onclose event of GPS port
	Dim objGDAbar
	Set objGDAbar = Application.Toolbars("GDA")
	If objGDAbar Is Nothing Then Exit Sub
	Application.Toolbars("GDA").Item("btnStaAtGPS").Enabled = False
End Sub

'****************************Application events***************************
Sub SavePreferences 'OnPreferencesChanged event of application
  Dim blnSuccess
  blnSuccess = Application.Properties("Load")
  MsgBox "User preferences have been changed"
End Sub