' coded by Evan Thoms, U.S. Geological Survey, 2003-2006 
' for newer versions do a find/replace on the string 
' Map.Layers(<LayerName>) and change the version number 


'GDA 1.0
'Object specific prefix convention used in this code 
'obj = object (form, page, or many controls) 
'cbo = combobox 
'ebo = editbox 
'lbo = listbox 
'btn = button 
'rs = recordset 
'f = system file 
'Variable data type prefix convention used in this code 
'str = string 
'int = integer 
'g_ = global variable 

Option Explicit 

' Global variables 
' Application level variables 
  Dim g_objDatum 
' Editform and Unit page 
  Dim g_objEF, g_objUnitCtls, g_objCurrXY, g_eboStaID 
  Dim g_rsSta, g_strStaID, g_strStaPref, g_strStaTime 
' Lithology page 
  Dim g_objLithCtls, g_strUnit 
' Structure page 
  Dim g_rsStruct, g_objStrucCtls 
' Samples page 
  Dim g_objSampCtls 
' Photos page 
  Dim g_objPhtCtls, g_strPhtPref, g_intNextPhtNo 
  g_intNextPhtNo = 1 

' Picklist and subtable file paths 
  Dim DataPath, AppletsPath, SamplesPath, SampTypesPath ', LithPath 
  Dim PhotosPath, GeoListPath 
   DataPath = Application.Properties("DataPath") & "\" 
   AppletsPath = Application.Properties("AppletsPath") & "\" 
   SamplesPath = DataPath & "Samples_1.0.dbf" 
   GeoListPath = DataPath & "Geologists_1.0.dbf" 
   SampTypesPath = DataPath & "SampTypes_1.0.dbf" 
   PhotosPath = DataPath & "Photos_1.0.dbf" 

'*******EDITFORM and UNIT PAGE****************** 
'----------------------------------------------- 
Sub InitializeForm 'OnLoad event of the editform. 
  Application.WaitCursor(-1) 
  Dim strGeologist 

  Set g_rsSta = Map.SelectionLayer.Records 
  g_rsSta.Bookmark = Map.SelectionBookmark 

' Get the coordinates in case a structure point is to be added 
  Set g_objCurrXY = Application.CreateAppObject("Point") 
  g_objCurrXY.X = g_rsSta.Fields.Shape.X 
  g_objCurrXY.Y = g_rsSta.Fields.Shape.Y 

  Set g_objEF = ThisEvent.Object 
  Set g_objUnitCtls = g_objEF.Pages("pgUnit").Controls 

' Set the references for the controls on all pages within this form 
  Set g_objLithCtls = g_objEF.Pages("pgLith").Controls 
  Set g_objStrucCtls = g_objEF.Pages("pgStr").Controls 
  Set g_objSampCtls = g_objEF.Pages("pgSamp").Controls 
  Set g_objPhtCtls = g_objEF.Pages("pgPhoto").Controls 
  Set g_eboStaID = g_objUnitCtls("eboStaID") 

' Set some controls to be invisible 
  g_eboStaID.Visible = False 
  g_objUnitCtls("eboDatumID").Visible = False
  g_objUnitCtls("eboMapPrj").Visible = False

' Initialize the form controls depending on whether the form is 
' associated with a new feature or an existing feature 
' Form modes: 0 = unbound form, 1 = identity, 2 = edit existing, 3 = new feature 
  If g_objEF.Mode = 3 Then 
' Get the previously used Geologist prefix if the layer has old records 
    If g_rsSta.RecordCount > 0 Then 
      g_rsSta.MoveLast 'Actually an empty record at EOF 
      g_rsSta.MovePrevious 
      g_strStaPref = Right(g_rsSta.Fields("STAID").Value, 2) 
      If IsNumeric(g_strStaPref) = True Then 
        g_strStaPref = "" 
      End If 
        g_objUnitCtls("cboGeologist").Value = g_strStaPref ' strGeologist 
      Else 
' Otherwise the prefix is nothing 
      g_strStaPref = "" 
    End If 
    Call PrefixChange 

' Make the station ID string and store the station time for edits 
    g_strStaID = GetDateString & g_strStaPref 
    g_strStaTime = GetDateString 
    g_eboStaID.Value = g_strStaID 
    Application.UserProperties("StationID") = g_strStaID 
' Get the datum id if this station is dependent on another 
    g_objUnitCtls("eboDatumID").Value = Application.UserProperties("DATUMID") 
  Else 

' Get the datetime string in case the geologist prefix is changed later. 
    g_strStaID = g_eboStaID.Value 
    g_strStaTime = Mid(g_strStaID, 3, 10) 

    g_objUnitCtls("eboDatumID").Value = g_rsSta.Fields("DATUM").Value 
  End If 

' Pick up location-by-offset property if set 
  If Application.UserProperties("OffsetLocation") = "Offset" Then 
    g_objUnitCtls("cboHowLoc").value = Application.UserProperties("OffsetLocation") 
  End If 

' Update the form caption to the new station ID 
  g_objEF.Caption = g_strStaID 

' Get the previously used mapunit 
  g_objUnitCtls("cboMapUnit").Value = g_rsSta.Fields("MAPUNIT").Value 

' Populate the hidden controls 
  g_objUnitCtls("eboSCALE").Value = Int(Application.Map.Scale) 
  g_objUnitCtls("cboMapUnit").Value = g_rsSta.Fields("MAPUNIT").Value
  g_objUnitCtls("eboMapPrj").Value = GetMapProjectionName
End Sub 

'----------------------------------------------- 
Sub AddUnit 'OnKillFocus of cboMapUnit 
  Dim Lithpath, cboUnit, fUnits, rsUnits 

  LithPath = Application.Properties("DataPath") & "\Units_1.0.dbf" 
  Set cboUnit = g_objUnitCtls("cboMapUnit") 
  Set fUnits = Application.CreateAppObject("File") 
  Set rsUnits = Application.CreateAppObject("Recordset") 
  If fUnits.Exists(LithPath) = False Then Exit Sub 
  Call rsUnits.Open (LithPath, 2) 

' Bail if the combobox is empty. 
  If cboUnit.Value = "" Then Exit Sub 

'if the dbf is empty, the movefirst method does not work 
  If rsUnits.RecordCount <> 0 Then 
    rsUnits.MoveFirst 
  End If 

  If rsUnits.Find ("[UNIT]=""" & cboUnit.Value &"""") = 0 Then 
    rsUnits.AddNew 
    rsUnits.Fields("UNIT").Value = cboUnit.Value 
    rsUnits.Update 
  End If 
  Map.Refresh 
End Sub 

'----------------------------------------------- 
Sub BuildGeologistList 'OnDropDown of cboGeologist 
  Dim cboGeo 
  Dim fGeoList, rsGeoList 

  Set cboGeo = ThisEvent.Object 
  Set fGeoList = Application.CreateAppObject("File") 
  Set rsGeoList = Application.CreateAppObject("Recordset") 

  If fGeoList.Exists(GeoListPath) = False Then 
    MsgBox "Geologists_1.0.dbf does not exist in the default data folder" 
    Exit Sub 
  End If 

  Call rsGeoList.Open (GeoListPath, 2) 
  cboGeo.Clear 
  cboGeo.AddItemsFromTable GeoListPath, "VALUE", "" 
End Sub 

'----------------------------------------------- 
Sub PrefixChange 'OnSelectChange event of cboGeologist 
  Dim cboGeologist 

  Set cboGeologist = g_objUnitCtls("cboGeologist") 
  g_strStaPref = cboGeologist.Value 
  g_strStaID = g_strStaTime & g_strStaPref 
  g_objEF.Caption = g_strStaID 
  g_eboStaID.Value = g_strStaID 
End Sub 

'----------------------------------------------- 
Sub AddGeologist 'OnKillFocus of cboGeologist 
  Dim cboGeologist, fGeoList, rsGeoList 

  Set cboGeologist = g_objUnitCtls("cboGeologist") 
  Set fGeoList = Application.CreateAppObject("File") 
  Set rsGeoList = Application.CreateAppObject("Recordset") 
  If fGeoList.Exists(GeoListPath) = False Then Exit Sub 
  Call rsGeoList.Open (GeoListPath, 2) 

' Bail if the combobox is empty. 
  If cboGeologist.Value = "" Then Exit Sub 

' If the dbf is empty, the movefirst method does not work 
  If rsGeoList.recordcount <> 0 Then 
    rsGeoList.MoveFirst 
  End If 

  If rsGeoList.Find ("[VALUE]=""" & cboGeologist.Value &"""") = 0 Then 
    rsGeoList.AddNew 
    rsGeoList.Fields("VALUE").Value = cboGeologist.Value 
    rsGeoList.Fields("TEXT").Value = cboGeologist.Value 
    rsGeoList.Update 
  End If 

  g_strStaPref = cboGeologist.Value 
  g_strStaID = g_strStaTime & g_strStaPref 
  g_objEF.Caption = g_strStaID 
  g_eboStaID.Value = g_strStaID 
End Sub 

'----------------------------------------------- 
Sub BuildUnitList 'OnDropDown of cboUnit 
'LithPath needs to be defined each time the combobox is accessed 
'because the user may have reset the Datapath in Options 
  Dim LithPath, fUnitsDBF, cboUnit 

  LithPath = Application.Properties("DataPath") & "\Units_1.0.dbf" 
  Set fUnitsDBF = Application.CreateAppObject("File") 
  Set cboUnit = g_objUnitCtls("cboMapUnit") 
  If fUnitsDBF.Exists(Lithpath) = False Then 
    Msgbox "Units_1.0.dbf does not exist in the default data folder" 
    Exit Sub 
  End If 
  cboUnit.Clear 
  Call cboUnit.AddItemsFromTable (LithPath, "UNIT", "UNIT") 
End Sub 

'----------------------------------------------- 
Sub AddNote 'OnClick event of btnAddNote 
  Dim fTextFile, TextPath, strTextBuff, strTextAll 
  Dim j 

  Set fTextFile = Application.CreateAppObject("File") 
  TextPath = Datapath & g_strStaID & ".txt" 
  fTextFile.Open TextPath, 8, 0 'open for read/write, open as ascii 
  strTextAll = "" 

' Comment out the following 6 lines if you would like new entries to start 
' at the end of the file, rather than at the beginning. 
  fTextFile.Seek = 0 'Start reading from the beginning 
  While Not fTextFile.EOF 
    strTextBuff = fTextFile.Readline & vbCrLf 
    strTextAll = strTextAll & strTextBuff 
  Wend 
  fTextFile.Seek = 0 'go back to beginning 

  fTextFile.Writeline Date & " " & FormatDateTime(Now, vbShortTime) & " " 
  fTextFile.Writeline 
  fTextFile.Write strTextAll 
  fTextFile.Close 
  Application.Run(TextPath) 
End Sub 

'----------------------------------------------- 
Sub AddSketch 'OnClick event of btnAddSketch 
  Dim fSketchFile, SketchPath 
  Set fSketchFile = Application.CreateAppObject("File") 

' Change the extension in the following line to create a graphics file for a 
' program other than Pocket Word (.psw extension) 
  SketchPath = Datapath & g_strStaID & ".psw" 
  If fSketchFile.Exists (SketchPath) = False Then 
    fSketchFile.Open SketchPath, 8, -2 'Open and write to the end, open with default 
    fSketchFile.Close 
  End If 
  Application.Run(SketchPath) 
End Sub 

'----------------------------------------------- 
Sub UnloadForm 'Free resources 
  Set g_objCurrXY = Nothing 
  Set g_objStrucCtls = Nothing 
  g_strStaID = "" 
  g_strStaPref = "" 
  Application.UserProperties("DATUMID") = "" 
  Application.UserProperties("StationID") = "" 
  Application.UserProperties("OffsetLocation") = "" 
End Sub 

'**************LITHOLOGY PAGE********************** 
'----------------------------------------------- 
Sub BuildLithologyList 'OnDropDown event of lithology combobox 
' If g_objEF.Fields("LITH").Value = "" Then 
  Dim Lithpath, cboLith, rsLith, strTempLith 
 
  LithPath = Application.Properties("DataPath") & "\Units_1.0.dbf" 
  Set cbolith = ThisEvent.Object 
  g_strUnit = g_objUnitCtls("cboMapUnit").Value 
  cboLith.Clear 
  Set rsLith = Application.CreateAppObject("recordset") 
  rsLith.Open LithPath, 2 

' if the dbf is empty, the movefirst method does not work 
  If rsLith.RecordCount <> 0 Then 
    rsLith.MoveFirst 
  End If 

' Would not need the following loop if the AddItemsFromTable method would work. 
' cboLith.AddItemsFromTable LithPath, "[LITH] ", " " , "[Unit]=""" & g_strUnit & """" 
  While Not rsLith.EOF 
    If rsLith.Fields("UNIT").Value = g_strUnit Then 
      Call cboLith.AddItem (rsLith.Fields("LITH").Value, "") 
    End If 
  rsLith.MoveNext 
  Wend 
  rsLith.Pack 
  rsLith.Close 
End Sub 

'----------------------------------------------- 
Sub NewLithology 'Onkillfocus event of cboLith 
'Checks for a new entry in the lithology combobox 
  Dim Lithpath, cboLith, rsLith, strUnitLabel 

  LithPath = Application.Properties("DataPath") & "\Units_1.0.dbf" 
  Set cboLith = ThisEvent.Object 
  Set rsLith = Application.CreateAppObject("Recordset") 
  Call rsLith.Open (LithPath, 2) 
  
' Bail if the combobox is empty. 
  If cboLith.Value = "" Then Exit Sub 
  
' Get the label for the chosen unit in case of AddNew 
  rsLith.Find "[UNIT]=""" & g_strUnit &"""" 
  strUnitLabel = rsLith.Fields("LABEL").Value 
  
' The find expression below searches for records where the lithology has already 
' been entered for the current mapunit choice. 
  rsLith.MoveFirst 
  If rsLith.Find ("[LITH]=""" & cboLith.Value &""" AND [UNIT]="""& g_strUnit &"""") = 0 Then 
    rsLith.AddNew 
    'Find way for LABEL field to be added to the new record 
    rsLith.Fields("UNIT").Value = g_strUnit 
    rsLith.Fields("LABEL").Value = strUnitLabel 
    rsLith.Fields("LITH").Value = cboLith.Value 
    rsLith.Update 
    rsLith.Pack 
    rsLith.Close 
  End If 
  g_rsSta.Fields("LITH").Value = cboLith.Value 
  g_rsSta.Update 
End Sub 

'*************STRUCTURE PAGE********************** 
'----------------------------------------------- 
Sub AddStructure 'OnClick event of the Add button on the structure page 
  Dim rsStruct, currLay 
  Dim rsSta 

' Check For the structure layer 
  Set currLay = Map.Layers("Structure_1.0") 
  If currLay Is Nothing Then 
    MsgBox "Add Structure_1.0.shp to the map." 
  Exit Sub 
  Else 
    Map.Layers("Structure_1.0").Editable = True 
  End If 

' Add a point at the location of the current station 
' Found that I have to set the structure point shape to the shape of a 
' new point that has the shape of the station point. I can't directly 
' set the station point shape to the structure point (?!) 
  Set g_rsStruct = Map.Layers("Structure_1.0").Records 
  Call g_rsStruct.AddNew 
  Set g_rsStruct.Fields.Shape = g_objCurrXY 
 
' Update the fields 
  g_rsStruct.Fields("STAID").Value = g_strStaID 
  g_rsStruct.Fields("STYPE").Value = g_objStrucCtls("cboSTYPE").Value 

' Get a type mismatch error if you don't write 0 to the field when the 
' editbox is empty (?!) 
  If g_objStrucCtls("eboAZI").Value = "" Then 
    g_rsStruct.Fields("AZI").Value = 0 
  Else 
    g_rsStruct.Fields("AZI").Value = g_objStrucCtls("eboAZI").Value 
  End If 

  If g_objStrucCtls("eboINC").Value = "" Then 
    g_rsStruct.Fields("INC").Value = 0 
  Else 
    g_rsStruct.Fields("INC").Value = g_objStrucCtls("eboINC").Value 
  End If 

  g_rsStruct.Fields("QUALITY").Value = g_objStrucCtls("chkQuality").Value 
  g_rsStruct.Fields("DATE_").Value = Now 
  g_rsStruct.Update 

' Update some controls 
  g_objStrucCtls("cboSTYPE").Value = "" 
  g_objStrucCtls("eboAZI").Value = "" 
  g_objStrucCtls("eboINC").Value = "" 
  g_objStrucCtls("chkQUALITY").Value = False 

' Return the editability to the stations layer 
  Map.Layers("Stations_1.0").Editable = True 
End Sub 

'----------------------------------------------- 
Sub UnloadStructurePage 'OnUnload event of structure page 
Set g_rsStruct = Nothing 
End Sub 

'*****************SAMPLE PAGE******************* 
'----------------------------------------------- 
Sub ActivateSamplePage 'OnSetActive event of pgSamp 
  Dim fFile, fDBFPath, strValue, strText, objLbo 

  Set objLbo = g_objSampCtls("lboSamp") 
  Set fFile = Application.CreateAppObject("File") 
  fDBFPath = SamplesPath 
  If fFile.Exists(fDBFPath) = False Then 
    MsgBox "Add the samples.dbf to the default data folder!" 
    Exit Sub 
  End If 
  g_objSampCtls("btnEdit").Enabled = True 
  strValue = "SAMPID" 
  strText = "purpose" 
  Call UpdateListBox (objlbo, fDBFPath, strValue, strText) 
End Sub 

'----------------------------------------------- 
Sub EditSamplesDBF 'OnClick event of the buttons (Add, Edit, Cancel) on sample page 
  Dim rsSamples, strSuffix, strSampID 

  Set rsSamples = Application.CreateAppObject("Recordset") 
  rsSamples.Open SamplesPath, 2 

' Depending on which button was clicked.... 
  Select Case ThisEvent.Object.Name 
  Case "btnAdd" 
    strSuffix = ReturnNextLetter(rsSamples) 
    rsSamples.AddNew 
    rsSamples.Fields("STAID").Value = g_strStaID 
    rsSamples.Fields("SAMPID").Value = g_strStaID & strSuffix 
    rsSamples.Fields("PURPOSE").Value = g_objSampCtls("cboSampType").Value 
    rsSamples.Update 
    rsSamples.Pack 
    rsSamples.Close 
  
  Case "btnEdit" 
    If g_objSampCtls("lboSamp").Value = "" Then Exit Sub 
    rsSamples.Find "[SAMPID]="""& g_objSampCtls("lboSamp").Value &"""" 
    rsSamples.Fields("PURPOSE").Value = g_objSampCtls("cboSampType").Value 
    rsSamples.Update 
    rsSamples.Pack 
    rsSamples.Close 

  Case "btnDelete" 
    If g_objSampCtls("lboSamp").Value = "" Then Exit Sub 
    rsSamples.MoveFirst 
    rsSamples.Find "[SAMPID]="""& g_objSampCtls("lboSamp").Value &"""" 
    strSampId = g_objSampCtls("lboSamp").Value 
    If MsgBox ("Are you sure you want to delete sample "& strSampID &" ?", vbYesNo) = vbNo Then Exit Sub 
    rsSamples.Delete 
    rsSamples.Update 
    rsSamples.Pack 
    rsSamples.Close 
  End Select 

  g_objSampCtls("lboSamp").Clear 
  Call ActivateSamplePage 
End Sub 

'**********************Photos Page******************************** 
'----------------------------------------------- 
Sub ActivatePhotoPage 'OnSetActive event of pgPhotos 
  Dim objLbo, fFile, fDBFPath, strValue, strText 

  Set objLbo = g_objPhtCtls("lboPhoto") 
  Set fFile = Application.CreateAppObject("File") 
  fDBFPath = PhotosPath 

  If fFile.Exists(fDBFPath) = False Then 
    MsgBox "Add Photos.dbf to the default data folder!" 
    objPgPhotos("btnAdd").Enabled = False 
    objPgPhotos("btnEdit").Enabled = False 
    objPgPhotos("btnDelete").Enabled = False 
    Exit Sub 
  End If 

' Prevent editboxes initializing to the previous entry 
  If g_objEF.Mode = 3 Then 
    g_objPhtCtls("eboWidth").Value = "" 
    g_objPhtCtls("eboTopic").Value = "" 
  End If 

  g_objPhtCtls("eboPhotoPref").Value = g_strPhtPref 
  g_objPhtCtls("btnAdd").Enabled = True 
  g_objPhtCtls("btnEdit").Enabled = True 
  g_objPhtCtls("btnDelete").Enabled = True 
  g_objPhtCtls("eboNextPhtNo").Value = g_intNextPhtNo 
  strValue = "PHOTOID" 
  strText = "TOPIC" 
  Call UpdateListBox (objlbo, fDBFPath, strValue, strText) 
End Sub 

'----------------------------------------------- 
Sub UpdatePhotoAttributes 'OnSelChange event of the photos listbox 
  Dim fFile, rsPhotos 

  Set fFile = Application.CreateAppObject("File") 
  Set rsPhotos = Application.CreateAppObject("Recordset") 
  If fFile.Exists(PhotosPath) = False Then Exit Sub 
  If g_objPhtCtls("lboPhoto").Value = "" Then Exit Sub 
  rsPhotos.Open PhotosPath, 2 
  rsPhotos.Find "[PHOTOID]="""& g_objPhtCtls("lboPhoto").Value &"""" 
  g_objPhtCtls("eboTopic").Value = rsPhotos.Fields("Topic") 
  g_objPhtCtls("eboWidth").Value = rsPhotos.Fields("Width") 
  g_objPhtCtls("cboOctant").Value = rsPhotos.Fields("Direction") 
  rsPhotos.Pack 
  rsPhotos.Close 
End Sub 

'----------------------------------------------- 
Sub EditPhotosDBF 'OnClick events of buttons on the photos page 
  Dim rsPhotos, strPhotoID 

Set rsPhotos = Application.CreateAppObject("Recordset") 
  rsPhotos.Open PhotosPath, 2 

' Depending on which button was clicked... 
  Select Case ThisEvent.Object.Name 
  Case "btnAdd" 
    g_strPhtPref = g_objPhtCtls("eboPhotoPref").Value 
    strPhotoID = CStr(g_strPhtPref & g_objPhtCtls("eboNextPhtNo").Value) & ".jpg" 
    rsPhotos.AddNew 
    rsPhotos.Fields("STAID").Value = g_strStaID 
    rsPhotos.Fields("PHOTOID").Value = strPhotoID 
    rsPhotos.Fields("TOPIC").Value = g_objPhtCtls("eboTopic").Value 
    rsPhotos.Fields("WIDTH").Value = g_objPhtCtls("eboWidth").Value 
    rsPhotos.Fields("DIRECTION").Value = g_objPhtCtls("cboOctant").Value 
    rsPhotos.Update 
    rsPhotos.Pack 
    rsPhotos.Close 
    g_intNextPhtNo = g_objPhtCtls("eboNextPhtNo").Value + 1 

  Case "btnEdit" 
    If g_objPhtCtls("lboPhoto").Value = "" Then Exit Sub 
    rsPhotos.Find "[PHOTOID]="""& g_objPhtCtls("lboPhoto").Value &"""" 
    rsPhotos.Fields("TOPIC").Value = g_objPhtCtls("eboTopic").Value 
    rsPhotos.Fields("WIDTH").Value = g_objPhtCtls("eboWidth").Value 
    rsPhotos.Fields("DIRECTION").Value = g_objPhtCtls("cboOctant").Value 
    rsPhotos.Update 
    rsPhotos.Pack 
    rsPhotos.Close 

  Case "btnDelete" 
    If g_objPhtCtls("lboPhoto").Value = "" Then Exit Sub 
    rsPhotos.MoveFirst 
    rsPhotos.Find "[PHOTOID]="""& g_objPhtCtls("lboPhoto").Value &"""" 
    strPhotoId = g_objPhtCtls("lboPhoto").Value 
    If MsgBox ("Are you sure you want to delete sample "& strPhotoID &" ?", vbYesNo) = vbNo Then Exit Sub 
    rsPhotos.Delete 
    rsPhotos.Update 
    rsPhotos.Pack 
    rsPhotos.Close 
  End Select 

  g_objPhtCtls("lboPhoto").Clear 
  Call ActivatePhotoPage 
End Sub 

'*************UTILITY FUNCTIONS************* 
'----------------------------------------------- 
Function FindGeologist(g_strStaPref) 
  Dim rsGeol 

  Set rsGeol = Application.CreateAppObject("Recordset") 
  Call rsGeol.Open (Datapath & "Geologists_1.0.dbf", 1) 
  rsGeol.MoveFirst 
  While Not rsGeol.EOF 
    If g_strStaPref = rsGeol.Fields("Value") Then 
      FindGeologist = rsGeol.Fields("Text").Value 
    Else FindGeologist = "Who" 
    End If 
    rsGeol.Movenext 
  Wend 
End Function 

'----------------------------------------------- 
Function GetDateString 
  Dim dtDate, dtTime 
  Dim intY, intM, intD, intH, intMin 

  dtDate = FormatDateTime(Now, vbShortDate) 
  dtTime = FormatDateTime(Now, vbLongTime) 
  intY = Right(Year(dtDate), 2) 

  intM = Month(dtDate) 
  If Len(intM) = 1 Then 
    intM = "0" & intM 
  End If 

  intD = Day(dtDate) 
  If Len(intD) = 1 Then 
    intD = "0" & intD 
  End If 

  intH = Hour(dtTime) 
  If Len(intH) = 1 Then 
    intH = "0" & intH 
  End If 

  intMin = Minute(dtTime) 
  If Len(intMin) = 1 Then 
    intMin = "0" & intMin 
  End If 

  GetDateString = intY & "_" & intM & intD & "_" & intH & intMin 
End Function 

'----------------------------------------------- 
Function ReturnNextLetter (rsSamples) 'Function to return the next letter in the 
'alphabet. After 26 lower case letters the function will return the '{' character, 
'which is the ASCII character after 'z'. The routine may be updated later to accomodate 
'more than 26 samples if it is reported as a problem. 
  Dim strCurrVal, strMaxVal 

'If the dbf is empty or there are no records for the current station, then 
'initialize with "a" 
  If rsSamples.RecordCount = 0 Or rsSamples.Find ("[StaID]="""& g_strStaID &"""") = 0 Then 
    ReturnNextLetter = "a" 
    Exit Function 
  End If 
  rsSamples.MoveFirst 
  strMaxVal = "a" 
  While Not rsSamples.EOF 
    If rsSamples.Fields("STAID").Value = g_strStaID Then 
      strCurrVal = Right(rsSamples.Fields("SAMPID").Value, 1) 
        If (strCurrVal > strMaxVal) Then 
          strMaxVal = strCurrVal 
        End If 
    End If 
    rsSamples.MoveNext 
  Wend 
  strMaxVal = Asc(strMaxVal) + 1 
  strMaxVal = Chr(strMaxVal) 
  ReturnNextLetter = strMaxVal 
End Function 

'------------------------------------------------------------------------------------------
Function GetMapProjectionName ' returns the name of the current map projection
  Dim objCS 
  'Set objCS = Application.CreateAppObject("CoordSys")
  Set objCS = Map.CoordinateSystem
  GetMapProjectionName = objCS.ProjectionName
End Function

'Following function not being used right now 3/15/04 
'----------------------------------------------- 
Function ReturnNextID (objRS, strFieldName) 
  Dim intMax 
  Dim intCurrVal 
  If objRS.RecordCount = 0 Then 
    ReturnNextID = 1 
    Exit Function 
  End If 
  'Get the first record 
  objRS.MoveFirst 
  'Initialize the max value to the first record 
  intMax = CInt(objRS.Fields(strFieldName).Value) 
  'Loop through the records, updating the max value if necessary 
  While Not objRS.EOF 
    intCurrVal = CInt(objRS.Fields(strFieldName).Value) 
    If (intCurrVal > intMax) Then 
      intMax = intCurrVal 
    End If 
    objRS.MoveNext 
  Wend 
  ReturnNextID = intMax + 1 
End Function 

'----------------------------------------------- 
' Generic sub to populate a listbox with records from a DBF where 
' the STAID field matches the current station ID. 
' Had trouble for awhile using the AddItemsFromTable method, so I 
' wrote this. 
Sub UpdateListBox(objLbo, fDBFPath, strValue, strText) 
  Dim rsListDBF 
  Set rsListDBF = Application.CreateAppObject("Recordset") 
  objLbo.Clear 
  Call rsListDBF.Open (fDBFPath, 1) 
  If rsListDBF.RecordCount = 0 Then Exit Sub 
    rsListDBF.MoveFirst 
  While Not rsListDBF.EOF 
    If rsListDBF.Fields("STAID").Value = g_strStaID Then 
      Call objLbo.AddItem (rsListDBF.Fields(strValue).Value, rsListDBF.Fields(strText).Value) 
    End If 
    rsListDBF.MoveNext 
  Wend 
  rsListDBF.Pack 
  rsListDBF.Close 
End Sub